package com.ensign.crm.module.crm.job;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ensign.crm.framework.quartz.core.handler.JobHandler;
import com.ensign.crm.framework.tenant.core.aop.TenantIgnore;
import com.ensign.crm.module.crm.exception.AllKingdeeException;
import com.ensign.crm.module.crm.service.ProxyService;
import com.ensign.crm.module.system.dal.dataobject.user.AdminUserDO;
import com.ensign.crm.module.system.dal.mysql.user.AdminUserMapper;
import com.ensign.crm.module.system.service.user.AdminUserService;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: TODO
 * @Date: 2024/9/26 14:06
 * @Created: by ZZSLL
 */

@Slf4j
@Component
public class UserSyncKingdeeTask implements JobHandler {


    @Autowired
    private AdminUserMapper userMapper;

    @Autowired
    private ProxyService proxyService;

    @Override
    @TenantIgnore
    public String execute(String param) throws Exception {
        LambdaQueryWrapper<AdminUserDO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AdminUserDO::getStatus, 0)
                .eq(AdminUserDO::getIsSyncKingdee, 1);
        List<AdminUserDO> users = userMapper.selectList(queryWrapper);
        List<Map<String, Object>> list = new ArrayList<>();

        for (AdminUserDO user : users) {
            Map<String, Object> addUser = new HashMap<>();
            addUser.put("number", user.getUsername());
            addUser.put("name", user.getNickname());
            addUser.put("webuserid", user.getId());
            addUser.put("webdeptid", user.getDeptId());
            addUser.put("function", "B");
            addUser.put("phone", user.getMobile());
            list.add(addUser);
        }
        Map<String, Object> body = new HashMap<>();
        body.put("data", list);
        log.info("请求body：{}", body);
        log.info("请求{}条", users.size());
        JSONObject jsonObject = proxyService.doRequest("/ierp/kapi/v2/yem/yem_crmbasic/yem_crm_dealeruser/CRM_yem_crm_dealeruser", "POST", body, null);
        log.info("响应body：{}", jsonObject);
        if (jsonObject.containsKey("status") && jsonObject.containsKey("errorCode")) {
            if (jsonObject.getBoolean("status") && jsonObject.getInteger("errorCode") == 0) {
                if (jsonObject.containsKey("data")) {
                    JSONObject dataObject = jsonObject.getJSONObject("data");
                    if (dataObject.containsKey("successCount")) {
                        int successCount = dataObject.getInteger("successCount");
                        log.info("成功{}条", successCount);
                        return "success";
                    }
                }
            }
        }
        throw new AllKingdeeException("task error：" + jsonObject);
    }
}
